#pragma once

#include <base/signal.h>
#include <dataspace/capability.h>
#include <session/session.h>
#include <irq_session/capability.h>

namespace Counter {

	struct Session;

	class Buffer_too_small : public Genode::Exception { };
	class Hsm_error        : public Genode::Exception { };
	class Invalid_counter  : public Genode::Exception { };

	using Id = Genode::String<100>;

	class Counter_type;
}

class Counter::Counter_type
{
private:
	bool             _valid { false };
	Genode::uint64_t _value { 0 };
public:
	Counter_type() = default;
	Counter_type(Genode::uint64_t value) : _valid(true), _value(value) { }
	bool valid() const { return _valid; }
	void value(Genode::uint64_t value)
	{
		_valid = true;
		_value = value;
	}
	Genode::uint64_t value() const
	{
		if (!_valid) {
			throw Invalid_counter();
		}
		return _value;
	}
};

struct Counter::Session : Genode::Session
{
	/**
	 * \noapi
	 */
	static const char *service_name() { return "Counter"; }

	enum { CAP_QUOTA = 2 };

	virtual ~Session() { }

	virtual Id id() const = 0;

	virtual Counter_type get() const = 0;

	virtual Counter_type increment() = 0;

	/*******************
	 ** RPC interface **
	 *******************/

	GENODE_RPC_THROW(Rpc_id,        Id,           id,        GENODE_TYPE_LIST(Hsm_error, Buffer_too_small));
	GENODE_RPC_THROW(Rpc_get,       Counter_type, get,       GENODE_TYPE_LIST(Hsm_error));
	GENODE_RPC_THROW(Rpc_increment, Counter_type, increment, GENODE_TYPE_LIST(Hsm_error));

	GENODE_RPC_INTERFACE(Rpc_id, Rpc_get, Rpc_increment);
};

